import torch
from mimic_video.cosmos_predict import CosmosPredictWrapper

def test_rtc_forward():
    print("Testing forward with fixed video prefix...")
    wrapper = CosmosPredictWrapper(tiny = True, random_weights = True, train_fixed_video_prefix = True, train_fixed_video_prefix_max_delay = 2)
    wrapper.train()
    
    batch = 2
    frames = 4
    videos = torch.randn(batch, frames, 3, 64, 64)
    prompts = ["a video of a robot", "a video of a cat"]
    
    # forward should work without error
    hiddens = wrapper(videos, prompts = prompts)
    print("Forward successful.")
    assert hiddens is not None

def test_rtc_finetune():
    print("Testing finetune with fixed video prefix...")
    wrapper = CosmosPredictWrapper(tiny = True, random_weights = True)
    
    class DummyDataset(torch.utils.data.Dataset):
        def __init__(self, n=4):
            self.n = n
        def __len__(self):
            return self.n
        def __getitem__(self, idx):
            return torch.randn(4, 3, 64, 64), "dummy prompt"

    dataset = DummyDataset()
    
    # finetune should work with train_fixed_video_prefix_max_delay
    wrapper.finetune(
        dataset,
        epochs = 1,
        batch_size = 2,
        train_fixed_video_prefix_max_delay = 2
    )
    print("Finetune successful.")

def test_rtc_disabled():
    print("Testing forward and finetune with fixed video prefix disabled...")
    # forward
    wrapper = CosmosPredictWrapper(tiny = True, random_weights = True, train_fixed_video_prefix = False)
    wrapper.train()
    videos = torch.randn(2, 4, 3, 64, 64)
    hiddens = wrapper(videos, prompts = ["a", "b"])
    assert hiddens is not None
    print("Forward (disabled) successful.")

    # finetune
    class DummyDataset(torch.utils.data.Dataset):
        def __init__(self, n=2): self.n = n
        def __len__(self): return self.n
        def __getitem__(self, idx): return torch.randn(4, 3, 64, 64), "dummy"

    wrapper.finetune(DummyDataset(), epochs = 1, batch_size = 2, train_fixed_video_prefix_max_delay = 0)
    print("Finetune (disabled) successful.")

if __name__ == "__main__":
    try:
        test_rtc_forward()
        test_rtc_finetune()
        test_rtc_disabled()
        print("All tests passed!")
    except Exception as e:
        print(f"Test failed with error: {e}")
        import traceback
        traceback.print_exc()



"""
Test Cosmos2_5PredictWrapper
"""

import torch
from mimic_video import Cosmos2_5PredictWrapper, MimicVideo

def test_cosmos_2_5_wrapper():
    print("Testing Cosmos2_5PredictWrapper basic functionality...")
    batch_size = 1
    wrapper = Cosmos2_5PredictWrapper(random_weights = True, tiny = True, extract_layer = 1)

    assert isinstance(wrapper, Cosmos2_5PredictWrapper)
    print(f"Verified class: {type(wrapper).__name__}")

    video = torch.rand(batch_size, 5, 3, 64, 64)
    print(f'input: {video.shape}')

    hidden = wrapper(video)
    print(f'hidden_states: {hidden.shape}')

    assert hidden.ndim == 3
    print('Cosmos2_5PredictWrapper test success')

def test_cosmos_2_5_integration():
    print("\nTesting Cosmos2_5PredictWrapper integration with MimicVideo...")
    wrapper = Cosmos2_5PredictWrapper(random_weights = True, tiny = False, extract_layer = 1)
    
    model = MimicVideo(
        dim = 512,
        video_predict_wrapper = wrapper,
        depth = 2
    )

    video = torch.rand(1, 5, 3, 32, 32)
    actions = torch.randn(1, 32, 20)
    joint_state = torch.randn(1, 32)

    loss = model(
        video = video,
        actions = actions,
        joint_state = joint_state,
        prompts = 'test prompt'
    )

    print(f'Loss: {loss.item()}')
    assert loss.numel() == 1
    
    sampled_actions = model.sample(
        video = video,
        joint_state = joint_state,
        prompts = 'sample prompt'
    )
    
    print(f'Sampleed actions shape: {sampled_actions.shape}')
    assert sampled_actions.shape == (1, 32, 20)
    print('MimicVideo integration test success')

if __name__ == '__main__':
    test_cosmos_2_5_wrapper()
    test_cosmos_2_5_integration()



"""
Debug test for Cosmos Pipeline - isolate the pipeline bottleneck
"""

import torch
import time
import pytest

@pytest.fixture
def pipeline():
    return test_pipeline_init()

def test_pipeline_init():
    """Test only pipeline construction"""
    print("Testing Pipeline init...")
    
    from mimic_video.cosmos_predict import (
        TINY_VAE_CONFIG, TINY_TRANSFORMER_CONFIG, TINY_T5_CONFIG
    )
    from diffusers.models.autoencoders.autoencoder_kl_cosmos import AutoencoderKLCosmos
    from diffusers.models.transformers.transformer_cosmos import CosmosTransformer3DModel
    from diffusers.schedulers.scheduling_edm_euler import EDMEulerScheduler
    from diffusers import CosmosVideoToWorldPipeline
    from transformers import T5EncoderModel, T5TokenizerFast, T5Config
    
    start = time.time()
    transformer = CosmosTransformer3DModel(num_layers=2, **TINY_TRANSFORMER_CONFIG)
    print(f"  Transformer init: {time.time() - start:.2f}s")
    
    start = time.time()
    vae = AutoencoderKLCosmos(**TINY_VAE_CONFIG)
    print(f"  VAE init: {time.time() - start:.2f}s")
    
    start = time.time()
    scheduler = EDMEulerScheduler()
    print(f"  Scheduler init: {time.time() - start:.2f}s")
    
    start = time.time()
    t5_config = T5Config(**TINY_T5_CONFIG)
    text_encoder = T5EncoderModel(t5_config)
    tokenizer = T5TokenizerFast.from_pretrained("google-t5/t5-small")
    print(f"  T5 init: {time.time() - start:.2f}s")
    
    # Dummy safety checker
    class DummySafetyChecker:
        def __call__(self, *args, **kwargs):
            return [], False
        def to(self, *args, **kwargs):
            return self
        def check_text_safety(self, text):
            return True
        def check_video_safety(self, video):
            return True
    
    start = time.time()
    pipeline = CosmosVideoToWorldPipeline(
        transformer=transformer,
        vae=vae,
        scheduler=scheduler,
        text_encoder=text_encoder,
        tokenizer=tokenizer,
        safety_checker=DummySafetyChecker(),
    )
    print(f"  Pipeline construction: {time.time() - start:.2f}s")
    
    print("✓ Pipeline init passed!")
    return pipeline

def test_pipeline_call(pipeline):
    """Test the pipeline __call__ method"""
    print("\nTesting Pipeline __call__...")
    
    # Create a tiny video input
    video = torch.rand(1, 5, 3, 32, 32)  # B, T, C, H, W
    print(f"Input video shape: {video.shape}")
    
    start = time.time()
    with torch.inference_mode():
        result = pipeline(
            video=video,
            prompt="test",
            num_inference_steps=1,
            guidance_scale=1.0,
            output_type="latent",
        )
    print(f"Pipeline call took: {time.time() - start:.2f}s")
    
    print("✓ Pipeline call passed!")
    return result

if __name__ == "__main__":
    pipeline = test_pipeline_init()
    test_pipeline_call(pipeline)
    print("\n✓ All pipeline tests passed!")



"""
Test CosmosPredictWrapper
"""

import torch
from mimic_video import CosmosPredictWrapper

def test_batching():
    print("Testing batching support...")
    batch_size = 2
    wrapper = CosmosPredictWrapper(random_weights = True, tiny = True, extract_layer = 19, device = "cpu")

    video = torch.rand(batch_size, 9, 3, 64, 64)
    print(f'input: {video.shape}')

    hidden = wrapper(video, prompts = ['a cat', 'a dog'])
    print(f'hidden_states: {hidden.shape}')

    assert hidden.shape == (batch_size, 192, 16)
    print('batching test success')

def test_single():
    print("\nTesting single video support...")
    wrapper = CosmosPredictWrapper(random_weights = True, tiny = True, extract_layer = 19, device = "cpu")

    video = torch.rand(1, 9, 3, 64, 64)
    print(f'input: {video.shape}')

    hidden = wrapper(video)
    print(f'hidden_states: {hidden.shape}')

    assert hidden.shape == (1, 192, 16)
    print('single test success')

def test_prompt_token_ids():
    print("\nTesting prompt_token_ids support...")
    batch_size = 2
    wrapper = CosmosPredictWrapper(random_weights = True, tiny = True, extract_layer = 19, device = "cpu")

    video = torch.rand(batch_size, 9, 3, 64, 64)
    print(f'input: {video.shape}')

    # Mock token ids
    prompt_token_ids = torch.randint(0, 32128, (batch_size, 16))

    hidden = wrapper(video, prompt_token_ids = prompt_token_ids)
    print(f'hidden_states: {hidden.shape}')

    assert hidden.shape == (batch_size, 192, 16)
    print('prompt_token_ids test success')

if __name__ == '__main__':
    test_batching()
    test_single()
    test_prompt_token_ids()



"""
Test CosmosPredictWrapper with denoising steps on CPU
"""

import torch
import time

def test_cosmos_denoising():
    print("Testing CosmosPredictWrapper with denoising steps...")
    
    from mimic_video import CosmosPredictWrapper
    
    start = time.time()
    wrapper = CosmosPredictWrapper(
        device = "cpu",
        torch_dtype = torch.float32,
        extract_layer = 1,
        random_weights = True,
        tiny = True
    )
    print(f"Init took: {time.time() - start:.2f}s")
    print(f"Wrapper dim_latent_hidden: {wrapper.dim_latent_hidden}")
    
    # Create a tiny dummy video
    batch_size = 1
    num_frames = 5
    video = torch.rand(batch_size, num_frames, 3, 32, 32)
    
    print(f"Input video shape: {video.shape}")
    
    # Test with 1 step
    print("\n--- 1 denoising step ---")
    start = time.time()
    hidden_1 = wrapper(videos=video, prompts="test", num_inference_steps=1)
    print(f"Forward took: {time.time() - start:.2f}s")
    print(f"Hidden states shape: {hidden_1.shape}")
    
    # Test with 4 steps
    print("\n--- 4 denoising steps ---")
    start = time.time()
    hidden_4 = wrapper(videos=video, prompts="test", num_inference_steps=4)
    print(f"Forward took: {time.time() - start:.2f}s")
    print(f"Hidden states shape: {hidden_4.shape}")
    print(f"Number of cached states: {len(wrapper.cached_hidden_states)}")
    
    assert hidden_1.shape[-1] == wrapper.dim_latent_hidden
    assert hidden_4.shape[-1] == wrapper.dim_latent_hidden
    
    print("\n✓ Denoising test passed!")

if __name__ == "__main__":
    test_cosmos_denoising()



import torch

# video wrapper
# but will be agnostic to the model

from mimic_video.cosmos_predict import CosmosPredictWrapper

video_wrapper = CosmosPredictWrapper(
    extract_layer = 1,
    random_weights = True,
    tiny = True
)

# mimic video

from mimic_video import MimicVideo

model = MimicVideo(512, video_wrapper)

# states

video = torch.rand(2, 5, 3, 32, 32)

joint_state = torch.randn(2, 32)

# action

actions = torch.randn(2, 32, 20)

# training

loss = model(
    prompts = [
        'put the package on the conveyer belt',
        'pass the butter'
    ],
    video = video,
    actions = actions,
    joint_state = joint_state
)

loss.backward()

# inference

actions = model.sample(
    prompts = 'peel the orange',
    video = video[:1],
    joint_state = joint_state[:1]
)

assert actions.shape == (1, 32, 20)